#ifndef KWIN_BLACKBOX_H
#define KWIN_BLACKBOX_H

#include <kwin/kwinbutton.h>
#include <kwin/client.h>

// bevel options
#define BImage_Flat		(1l<<1)
#define BImage_Sunken		(1l<<2)
#define BImage_Raised		(1l<<3)

// textures
#define BImage_Solid		(1l<<4)
#define BImage_Gradient		(1l<<5)

// gradients
#define BImage_Horizontal	(1l<<6)
#define BImage_Vertical		(1l<<7)
#define BImage_Diagonal		(1l<<8)
#define BImage_CrossDiagonal	(1l<<9)
#define BImage_Rectangle	(1l<<10)
#define BImage_Pyramid		(1l<<11)
#define BImage_PipeCross	(1l<<12)
#define BImage_Elliptic		(1l<<13)

// bevel types
#define BImage_Bevel1		(1l<<14)
#define BImage_Bevel2		(1l<<15)

// inverted image
#define BImage_Invert		(1l<<16)

// parent relative image
#define BImage_ParentRelative   (1l<<17)

// fake interlaced image
#  define BImage_Interlaced	(1l<<18)

#include <qlayout.h> 

#include <kpixmap.h>

class QSpacerItem;
class QBoxLayout;

namespace KWinInternal
{

	enum Buttons
	{ BtnMax = 0, BtnMin, BtnClose, BtnNone };


	class BlackboxButton: public QSpacerItem
	{
	  public:
		BlackboxButton (Client * parent = 0, const char *name =
			0, const Buttons type = BtnMax, int size =
			14, KPixmap * pixmaps[3] = 0);
		QSize sizeHint () const;
		void drawButton (QPainter * p);
		KPixmap *pixButton[3];
		Client *client;			// owner
		Buttons m_type;			// button type
		bool m_down;
		bool m_hide;
		int      last_button;
		int m_size;
		inline void hide (void)
		{
			m_hide = false;
		}
		inline void show (void)
		{
			m_hide = true;
		}
		inline bool isVisible(void){
			return (m_hide);
		}
		inline void press(void){
			m_down=true;
		}
		inline void unpress(void){
			m_down=false;
		}
	};

	class BlackboxClient:public KWinInternal::Client
	{
	  Q_OBJECT public:
		 BlackboxClient (Workspace *, WId, QWidget * parent = 0, const char *name =
			0);
		~BlackboxClient ()
		{						/* empty */
		};

	  protected:
		// Events

		void resizeEvent (QResizeEvent *);
		void mousePressEvent( QMouseEvent * );
		void paintEvent (QPaintEvent *);
		void mouseDoubleClickEvent (QMouseEvent *);
		void captionChange (const QString &);
		//void stickyChange (bool on);
		void shadeChange(bool shaded);
		//void maximizeChange (bool);
		void addClientButtons (const QString & s);
		void activeChange (bool);
		void makeButtons ();
		// Return mouse position
		void mouseReleaseEvent (QMouseEvent *);
		MousePosition mousePosition (const QPoint &) const;


		protected slots:void slotReset ();
		void slotMaximize ();
/*
	  signals:
		void stkyChange (bool);
		void maxChange (bool);
*/
	  private:
		bool smallButtons;
		int titleHeight;
		// titlebar widget
		QSpacerItem *titlebar;
		// title bar buttons
		BlackboxButton *button[3];
		// layouts
		QVBoxLayout *mainLayout;
		QHBoxLayout *titleLayout;
		QHBoxLayout *windowLayout;
		QVBoxLayout *botLayout;

		QLayoutItem *botSpacerItem;
		void setupLayout ();
		QFont m_font;
		QFontMetrics *m_fm;
		KPixmap *pixTitle[2];
		KPixmap *pixLabel[2];
		KPixmap *pixHandle[2];
		KPixmap *pixButton[3];
		KPixmap *pixGrip[2];
		bool pixmade;
		bool madeButtons;
		int lastButton;
	};

}

#endif
